import {
  Controller,
  Get,
  Post,
  Body,
  Patch,
  Param,
  Delete,
  HttpCode,
  HttpStatus,
  Query,
} from '@nestjs/common';
import { ProjectsService } from '../services/projects.service';
import { CreateProjectDto } from '../dto/create-project.dto';
import { UpdateProjectDto } from '../dto/update-project.dto';

@Controller('projects')
export class ProjectsController {
  constructor(private readonly projectsService: ProjectsService) {}

  /**
   * Create a new project
   */
  @Post()
  @HttpCode(HttpStatus.CREATED)
  create(@Body() createProjectDto: CreateProjectDto) {
    return this.projectsService.create(createProjectDto);
  }

  /**
   * Get all projects or filter by owner ID
   */
  @Get()
  findAll(@Query('ownerId') ownerId?: string) {
    if (ownerId) {
      return this.projectsService.findByOwnerId(ownerId);
    }
    return this.projectsService.findAll();
  }

  /**
   * Get a project by ID
   */
  @Get(':id')
  findOne(@Param('id') id: string) {
    return this.projectsService.findById(id);
  }

  /**
   * Update a project
   */
  @Patch(':id')
  update(@Param('id') id: string, @Body() updateProjectDto: UpdateProjectDto) {
    return this.projectsService.update(id, updateProjectDto);
  }

  /**
   * Delete a project
   */
  @Delete(':id')
  @HttpCode(HttpStatus.NO_CONTENT)
  remove(@Param('id') id: string) {
    return this.projectsService.remove(id);
  }

  /**
   * Check if a user has access to a project
   */
  @Get(':id/check-access/:userId')
  checkUserAccess(@Param('id') id: string, @Param('userId') userId: string) {
    return this.projectsService.checkUserAccess(id, userId);
  }

  /**
   * Add a collaborator to a project
   */
  @Post(':id/collaborators/:userId')
  @HttpCode(HttpStatus.CREATED)
  addCollaborator(@Param('id') id: string, @Param('userId') userId: string) {
    return this.projectsService.addCollaborator(id, userId);
  }

  /**
   * Remove a collaborator from a project
   */
  @Delete(':id/collaborators/:userId')
  @HttpCode(HttpStatus.NO_CONTENT)
  removeCollaborator(@Param('id') id: string, @Param('userId') userId: string) {
    return this.projectsService.removeCollaborator(id, userId);
  }

  /**
   * Get all collaborators for a project
   */
  @Get(':id/collaborators')
  getCollaborators(@Param('id') id: string) {
    return this.projectsService.getCollaborators(id);
  }
}
