import { Test, TestingModule } from '@nestjs/testing';
import { ProjectsService } from './projects.service';
import { NotFoundException } from '@nestjs/common';
import { DRIZZLE } from '../../../database/database.module';

describe('ProjectsService', () => {
  let service: ProjectsService;
  let mockDb: any;

  // Mock data
  const mockProject = {
    id: 'project1',
    name: 'Test Project',
    description: 'Test Description',
    ownerId: 'user1',
    settings: {},
    createdAt: new Date(),
    updatedAt: new Date(),
  };

  const mockUser = {
    id: 'user2',
    name: 'Test User',
    githubId: '12345',
    createdAt: new Date(),
    updatedAt: new Date(),
  };

  const mockCollaboration = {
    id: 'collab1',
    projectId: 'project1',
    userId: 'user2',
    createdAt: new Date(),
  };

  // Mock database operations
  const mockDbOperations = {
    select: jest.fn().mockReturnThis(),
    from: jest.fn().mockReturnThis(),
    where: jest.fn().mockReturnThis(),
    insert: jest.fn().mockReturnThis(),
    values: jest.fn().mockReturnThis(),
    update: jest.fn().mockReturnThis(),
    set: jest.fn().mockReturnThis(),
    delete: jest.fn().mockReturnThis(),
    innerJoin: jest.fn().mockReturnThis(),
    returning: jest.fn().mockImplementation(() => {
      return [mockProject];
    }),
  };

  beforeEach(async () => {
    mockDb = {
      ...mockDbOperations,
    };

    const module: TestingModule = await Test.createTestingModule({
      providers: [
        ProjectsService,
        {
          provide: DRIZZLE,
          useValue: mockDb,
        },
      ],
    }).compile();

    service = module.get<ProjectsService>(ProjectsService);
  });

  afterEach(() => {
    jest.clearAllMocks();
  });

  it('should be defined', () => {
    expect(service).toBeDefined();
  });

  describe('create', () => {
    it('should create a new project', async () => {
      const createProjectDto = {
        name: 'Test Project',
        description: 'Test Description',
        ownerId: 'user1',
      };

      const result = await service.create(createProjectDto);

      expect(mockDb.insert).toHaveBeenCalled();
      expect(mockDb.values).toHaveBeenCalledWith(createProjectDto);
      expect(result).toEqual(mockProject);
    });
  });

  describe('findAll', () => {
    it('should return all projects', async () => {
      mockDb.select.mockImplementationOnce(() => mockDbOperations);
      mockDbOperations.from.mockImplementationOnce(() => [mockProject]);

      const result = await service.findAll();

      expect(mockDb.select).toHaveBeenCalled();
      expect(mockDb.from).toHaveBeenCalled();
      expect(result).toEqual([mockProject]);
    });
  });

  describe('findByOwnerId', () => {
    it('should return projects for a specific owner', async () => {
      const ownerId = 'user1';
      mockDb.select.mockImplementationOnce(() => mockDbOperations);
      mockDbOperations.from.mockImplementationOnce(() => mockDbOperations);
      mockDbOperations.where.mockImplementationOnce(() => [mockProject]);

      const result = await service.findByOwnerId(ownerId);

      expect(mockDb.select).toHaveBeenCalled();
      expect(mockDb.from).toHaveBeenCalled();
      expect(mockDb.where).toHaveBeenCalled();
      expect(result).toEqual([mockProject]);
    });
  });

  describe('findById', () => {
    it('should return a project by ID', async () => {
      const id = 'project1';
      mockDb.select.mockImplementationOnce(() => mockDbOperations);
      mockDbOperations.from.mockImplementationOnce(() => mockDbOperations);
      mockDbOperations.where.mockImplementationOnce(() => [mockProject]);

      const result = await service.findById(id);

      expect(mockDb.select).toHaveBeenCalled();
      expect(mockDb.from).toHaveBeenCalled();
      expect(mockDb.where).toHaveBeenCalled();
      expect(result).toEqual(mockProject);
    });

    it('should throw NotFoundException if project not found', async () => {
      const id = 'nonexistent';
      mockDb.select.mockImplementationOnce(() => mockDbOperations);
      mockDbOperations.from.mockImplementationOnce(() => mockDbOperations);
      mockDbOperations.where.mockImplementationOnce(() => []);

      await expect(service.findById(id)).rejects.toThrow(NotFoundException);
    });
  });

  describe('update', () => {
    it('should update a project', async () => {
      const id = 'project1';
      const updateProjectDto = {
        name: 'Updated Project',
      };

      const result = await service.update(id, updateProjectDto);

      expect(mockDb.update).toHaveBeenCalled();
      expect(mockDb.set).toHaveBeenCalled();
      expect(mockDb.where).toHaveBeenCalled();
      expect(result).toEqual(mockProject);
    });

    it('should throw NotFoundException if project not found', async () => {
      const id = 'nonexistent';
      const updateProjectDto = {
        name: 'Updated Project',
      };

      mockDb.update.mockImplementationOnce(() => mockDbOperations);
      mockDbOperations.set.mockImplementationOnce(() => mockDbOperations);
      mockDbOperations.where.mockImplementationOnce(() => mockDbOperations);
      mockDbOperations.returning.mockImplementationOnce(() => []);

      await expect(service.update(id, updateProjectDto)).rejects.toThrow(NotFoundException);
    });
  });

  describe('remove', () => {
    it('should delete a project', async () => {
      const id = 'project1';

      const result = await service.remove(id);

      expect(mockDb.delete).toHaveBeenCalled();
      expect(mockDb.where).toHaveBeenCalled();
      expect(result).toEqual(mockProject);
    });

    it('should throw NotFoundException if project not found', async () => {
      const id = 'nonexistent';

      mockDb.delete.mockImplementationOnce(() => mockDbOperations);
      mockDbOperations.where.mockImplementationOnce(() => mockDbOperations);
      mockDbOperations.returning.mockImplementationOnce(() => []);

      await expect(service.remove(id)).rejects.toThrow(NotFoundException);
    });
  });

  describe('checkUserAccess', () => {
    it('should return true if user is the owner of the project', async () => {
      const projectId = 'project1';
      const userId = 'user1';

      // Mock owner check
      mockDb.select.mockImplementationOnce(() => mockDbOperations);
      mockDbOperations.from.mockImplementationOnce(() => mockDbOperations);
      mockDbOperations.where.mockImplementationOnce(() => [mockProject]);

      const result = await service.checkUserAccess(projectId, userId);

      expect(mockDb.select).toHaveBeenCalled();
      expect(mockDb.from).toHaveBeenCalled();
      expect(mockDb.where).toHaveBeenCalled();
      expect(result).toBe(true);
    });

    it('should return true if user is a collaborator on the project', async () => {
      const projectId = 'project1';
      const userId = 'user2';

      // Mock owner check
      mockDb.select.mockImplementationOnce(() => mockDbOperations);
      mockDbOperations.from.mockImplementationOnce(() => mockDbOperations);
      mockDbOperations.where.mockImplementationOnce(() => []);

      // Mock collaborator check
      mockDb.select.mockImplementationOnce(() => mockDbOperations);
      mockDbOperations.from.mockImplementationOnce(() => mockDbOperations);
      mockDbOperations.where.mockImplementationOnce(() => [mockCollaboration]);

      const result = await service.checkUserAccess(projectId, userId);

      expect(mockDb.select).toHaveBeenCalledTimes(2);
      expect(mockDb.from).toHaveBeenCalledTimes(2);
      expect(mockDb.where).toHaveBeenCalledTimes(2);
      expect(result).toBe(true);
    });

    it('should return false if user does not have access to project', async () => {
      const projectId = 'project1';
      const userId = 'user3';

      // Mock owner check
      mockDb.select.mockImplementationOnce(() => mockDbOperations);
      mockDbOperations.from.mockImplementationOnce(() => mockDbOperations);
      mockDbOperations.where.mockImplementationOnce(() => []);

      // Mock collaborator check
      mockDb.select.mockImplementationOnce(() => mockDbOperations);
      mockDbOperations.from.mockImplementationOnce(() => mockDbOperations);
      mockDbOperations.where.mockImplementationOnce(() => []);

      const result = await service.checkUserAccess(projectId, userId);

      expect(mockDb.select).toHaveBeenCalledTimes(2);
      expect(mockDb.from).toHaveBeenCalledTimes(2);
      expect(mockDb.where).toHaveBeenCalledTimes(2);
      expect(result).toBe(false);
    });
  });

  describe('addCollaborator', () => {
    it('should add a collaborator to a project', async () => {
      const projectId = 'project1';
      const userId = 'user2';

      // Mock findById
      jest.spyOn(service, 'findById').mockResolvedValueOnce(mockProject);

      // Mock user check
      mockDb.select.mockImplementationOnce(() => mockDbOperations);
      mockDbOperations.from.mockImplementationOnce(() => mockDbOperations);
      mockDbOperations.where.mockImplementationOnce(() => [mockUser]);

      // Mock relation check
      mockDb.select.mockImplementationOnce(() => mockDbOperations);
      mockDbOperations.from.mockImplementationOnce(() => mockDbOperations);
      mockDbOperations.where.mockImplementationOnce(() => []);

      // Mock insert
      mockDb.insert.mockImplementationOnce(() => mockDbOperations);
      mockDbOperations.values.mockImplementationOnce(() => mockDbOperations);
      mockDbOperations.returning.mockImplementationOnce(() => [mockCollaboration]);

      const result = await service.addCollaborator(projectId, userId);

      expect(service.findById).toHaveBeenCalledWith(projectId);
      expect(mockDb.select).toHaveBeenCalledTimes(2);
      expect(mockDb.from).toHaveBeenCalledTimes(2);
      expect(mockDb.where).toHaveBeenCalledTimes(2);
      expect(mockDb.insert).toHaveBeenCalled();
      expect(mockDb.values).toHaveBeenCalledWith({
        projectId,
        userId,
      });
      expect(result).toEqual(mockCollaboration);
    });

    it('should return existing collaboration if user is already a collaborator', async () => {
      const projectId = 'project1';
      const userId = 'user2';

      // Mock findById
      jest.spyOn(service, 'findById').mockResolvedValueOnce(mockProject);

      // Mock user check
      mockDb.select.mockImplementationOnce(() => mockDbOperations);
      mockDbOperations.from.mockImplementationOnce(() => mockDbOperations);
      mockDbOperations.where.mockImplementationOnce(() => [mockUser]);

      // Mock relation check
      mockDb.select.mockImplementationOnce(() => mockDbOperations);
      mockDbOperations.from.mockImplementationOnce(() => mockDbOperations);
      mockDbOperations.where.mockImplementationOnce(() => [mockCollaboration]);

      const result = await service.addCollaborator(projectId, userId);

      expect(service.findById).toHaveBeenCalledWith(projectId);
      expect(mockDb.select).toHaveBeenCalledTimes(2);
      expect(mockDb.from).toHaveBeenCalledTimes(2);
      expect(mockDb.where).toHaveBeenCalledTimes(2);
      expect(mockDb.insert).not.toHaveBeenCalled();
      expect(result).toEqual(mockCollaboration);
    });

    it('should throw NotFoundException if user not found', async () => {
      const projectId = 'project1';
      const userId = 'nonexistent';

      // Mock findById
      jest.spyOn(service, 'findById').mockResolvedValueOnce(mockProject);

      // Mock user check
      mockDb.select.mockImplementationOnce(() => mockDbOperations);
      mockDbOperations.from.mockImplementationOnce(() => mockDbOperations);
      mockDbOperations.where.mockImplementationOnce(() => []);

      await expect(service.addCollaborator(projectId, userId)).rejects.toThrow(NotFoundException);
    });
  });

  describe('removeCollaborator', () => {
    it('should remove a collaborator from a project', async () => {
      const projectId = 'project1';
      const userId = 'user2';

      mockDb.delete.mockImplementationOnce(() => mockDbOperations);
      mockDbOperations.where.mockImplementationOnce(() => mockDbOperations);
      mockDbOperations.returning.mockImplementationOnce(() => [mockCollaboration]);

      const result = await service.removeCollaborator(projectId, userId);

      expect(mockDb.delete).toHaveBeenCalled();
      expect(mockDb.where).toHaveBeenCalled();
      expect(result).toEqual(mockCollaboration);
    });

    it('should throw NotFoundException if collaboration not found', async () => {
      const projectId = 'project1';
      const userId = 'nonexistent';

      mockDb.delete.mockImplementationOnce(() => mockDbOperations);
      mockDbOperations.where.mockImplementationOnce(() => mockDbOperations);
      mockDbOperations.where.mockImplementationOnce(() => mockDbOperations);
      mockDbOperations.returning.mockImplementationOnce(() => []);

      await expect(service.removeCollaborator(projectId, userId)).rejects.toThrow(NotFoundException);
    });
  });

  describe('getCollaborators', () => {
    it('should get all collaborators for a project', async () => {
      const projectId = 'project1';
      const mockCollaborators = [{ user: mockUser }];

      // Mock findById
      jest.spyOn(service, 'findById').mockResolvedValueOnce(mockProject);

      // Mock get collaborators
      mockDb.select.mockImplementationOnce(() => mockDbOperations);
      mockDbOperations.from.mockImplementationOnce(() => mockDbOperations);
      mockDbOperations.innerJoin.mockImplementationOnce(() => mockDbOperations);
      mockDbOperations.where.mockImplementationOnce(() => mockCollaborators);

      const result = await service.getCollaborators(projectId);

      expect(service.findById).toHaveBeenCalledWith(projectId);
      expect(mockDb.select).toHaveBeenCalled();
      expect(mockDb.from).toHaveBeenCalled();
      expect(mockDb.innerJoin).toHaveBeenCalled();
      expect(mockDb.where).toHaveBeenCalled();
    });
  });
});
