import {
  Controller,
  Get,
  Post,
  Body,
  Param,
  Delete,
  Put,
  UseGuards,
  Query,
} from '@nestjs/common';
import { TagsService } from '../services/tags.service';
import { CreateTagDto } from '../dto/create-tag.dto';
import { UpdateTagDto } from '../dto/update-tag.dto';
import { JwtAuthGuard } from '../../auth/guards/jwt-auth.guard';

@Controller('tags')
@UseGuards(JwtAuthGuard)
export class TagsController {
  constructor(private readonly tagsService: TagsService) {}

  /**
   * Create a new tag
   */
  @Post()
  create(@Body() createTagDto: CreateTagDto) {
    return this.tagsService.create(createTagDto);
  }

  /**
   * Get all tags or filter by type
   */
  @Get()
  findAll(@Query('type') type?: 'PROJECT' | 'PERSON') {
    if (type) {
      return this.tagsService.findByType(type);
    }
    return this.tagsService.findAll();
  }

  /**
   * Get a tag by ID
   */
  @Get(':id')
  findOne(@Param('id') id: string) {
    return this.tagsService.findById(id);
  }

  /**
   * Update a tag
   */
  @Put(':id')
  update(@Param('id') id: string, @Body() updateTagDto: UpdateTagDto) {
    return this.tagsService.update(id, updateTagDto);
  }

  /**
   * Delete a tag
   */
  @Delete(':id')
  remove(@Param('id') id: string) {
    return this.tagsService.remove(id);
  }

  /**
   * Add a tag to a person
   */
  @Post('persons/:personId/tags/:tagId')
  addTagToPerson(
    @Param('personId') personId: string,
    @Param('tagId') tagId: string,
  ) {
    return this.tagsService.addTagToPerson(tagId, personId);
  }

  /**
   * Remove a tag from a person
   */
  @Delete('persons/:personId/tags/:tagId')
  removeTagFromPerson(
    @Param('personId') personId: string,
    @Param('tagId') tagId: string,
  ) {
    return this.tagsService.removeTagFromPerson(tagId, personId);
  }

  /**
   * Get all tags for a person
   */
  @Get('persons/:personId/tags')
  getTagsForPerson(@Param('personId') personId: string) {
    return this.tagsService.getTagsForPerson(personId);
  }

  /**
   * Add a tag to a project
   */
  @Post('projects/:projectId/tags/:tagId')
  addTagToProject(
    @Param('projectId') projectId: string,
    @Param('tagId') tagId: string,
  ) {
    return this.tagsService.addTagToProject(tagId, projectId);
  }

  /**
   * Remove a tag from a project
   */
  @Delete('projects/:projectId/tags/:tagId')
  removeTagFromProject(
    @Param('projectId') projectId: string,
    @Param('tagId') tagId: string,
  ) {
    return this.tagsService.removeTagFromProject(tagId, projectId);
  }

  /**
   * Get all tags for a project
   */
  @Get('projects/:projectId/tags')
  getTagsForProject(@Param('projectId') projectId: string) {
    return this.tagsService.getTagsForProject(projectId);
  }
}