import { Injectable, NotFoundException, Inject } from '@nestjs/common';
import { eq, and } from 'drizzle-orm';
import { DRIZZLE } from '../../../database/database.module';
import * as schema from '../../../database/schema';
import { CreateTagDto } from '../dto/create-tag.dto';
import { UpdateTagDto } from '../dto/update-tag.dto';

@Injectable()
export class TagsService {
  constructor(@Inject(DRIZZLE) private readonly db: any) {}

  /**
   * Create a new tag
   */
  async create(createTagDto: CreateTagDto) {
    const [tag] = await this.db
      .insert(schema.tags)
      .values({
        ...createTagDto,
      })
      .returning();
    return tag;
  }

  /**
   * Find all tags
   */
  async findAll() {
    return this.db.select().from(schema.tags);
  }

  /**
   * Find tags by type
   */
  async findByType(type: 'PROJECT' | 'PERSON') {
    return this.db
      .select()
      .from(schema.tags)
      .where(eq(schema.tags.type, type));
  }

  /**
   * Find a tag by ID
   */
  async findById(id: string) {
    const [tag] = await this.db
      .select()
      .from(schema.tags)
      .where(eq(schema.tags.id, id));
    
    if (!tag) {
      throw new NotFoundException(`Tag with ID ${id} not found`);
    }
    
    return tag;
  }

  /**
   * Update a tag
   */
  async update(id: string, updateTagDto: UpdateTagDto) {
    const [tag] = await this.db
      .update(schema.tags)
      .set({
        ...updateTagDto,
        updatedAt: new Date(),
      })
      .where(eq(schema.tags.id, id))
      .returning();
    
    if (!tag) {
      throw new NotFoundException(`Tag with ID ${id} not found`);
    }
    
    return tag;
  }

  /**
   * Delete a tag
   */
  async remove(id: string) {
    const [tag] = await this.db
      .delete(schema.tags)
      .where(eq(schema.tags.id, id))
      .returning();
    
    if (!tag) {
      throw new NotFoundException(`Tag with ID ${id} not found`);
    }
    
    return tag;
  }

  /**
   * Add a tag to a person
   */
  async addTagToPerson(tagId: string, personId: string) {
    // Check if the tag exists and is of type PERSON
    const tag = await this.findById(tagId);
    if (tag.type !== 'PERSON') {
      throw new Error(`Tag with ID ${tagId} is not of type PERSON`);
    }
    
    // Check if the person exists
    const [person] = await this.db
      .select()
      .from(schema.persons)
      .where(eq(schema.persons.id, personId));
    
    if (!person) {
      throw new NotFoundException(`Person with ID ${personId} not found`);
    }
    
    // Check if the tag is already associated with the person
    const [existingRelation] = await this.db
      .select()
      .from(schema.personToTag)
      .where(
        and(
          eq(schema.personToTag.personId, personId),
          eq(schema.personToTag.tagId, tagId)
        )
      );
    
    if (existingRelation) {
      return existingRelation;
    }
    
    // Add the tag to the person
    const [relation] = await this.db
      .insert(schema.personToTag)
      .values({
        personId,
        tagId,
      })
      .returning();
    
    return relation;
  }

  /**
   * Remove a tag from a person
   */
  async removeTagFromPerson(tagId: string, personId: string) {
    const [relation] = await this.db
      .delete(schema.personToTag)
      .where(
        and(
          eq(schema.personToTag.personId, personId),
          eq(schema.personToTag.tagId, tagId)
        )
      )
      .returning();
    
    if (!relation) {
      throw new NotFoundException(`Tag with ID ${tagId} is not associated with person with ID ${personId}`);
    }
    
    return relation;
  }

  /**
   * Add a tag to a project
   */
  async addTagToProject(tagId: string, projectId: string) {
    // Check if the tag exists and is of type PROJECT
    const tag = await this.findById(tagId);
    if (tag.type !== 'PROJECT') {
      throw new Error(`Tag with ID ${tagId} is not of type PROJECT`);
    }
    
    // Check if the project exists
    const [project] = await this.db
      .select()
      .from(schema.projects)
      .where(eq(schema.projects.id, projectId));
    
    if (!project) {
      throw new NotFoundException(`Project with ID ${projectId} not found`);
    }
    
    // Check if the tag is already associated with the project
    const [existingRelation] = await this.db
      .select()
      .from(schema.projectToTag)
      .where(
        and(
          eq(schema.projectToTag.projectId, projectId),
          eq(schema.projectToTag.tagId, tagId)
        )
      );
    
    if (existingRelation) {
      return existingRelation;
    }
    
    // Add the tag to the project
    const [relation] = await this.db
      .insert(schema.projectToTag)
      .values({
        projectId,
        tagId,
      })
      .returning();
    
    return relation;
  }

  /**
   * Remove a tag from a project
   */
  async removeTagFromProject(tagId: string, projectId: string) {
    const [relation] = await this.db
      .delete(schema.projectToTag)
      .where(
        and(
          eq(schema.projectToTag.projectId, projectId),
          eq(schema.projectToTag.tagId, tagId)
        )
      )
      .returning();
    
    if (!relation) {
      throw new NotFoundException(`Tag with ID ${tagId} is not associated with project with ID ${projectId}`);
    }
    
    return relation;
  }

  /**
   * Get all tags for a person
   */
  async getTagsForPerson(personId: string) {
    // Check if the person exists
    const [person] = await this.db
      .select()
      .from(schema.persons)
      .where(eq(schema.persons.id, personId));
    
    if (!person) {
      throw new NotFoundException(`Person with ID ${personId} not found`);
    }
    
    // Get all tags for the person
    return this.db
      .select({
        tag: schema.tags,
      })
      .from(schema.personToTag)
      .innerJoin(schema.tags, eq(schema.personToTag.tagId, schema.tags.id))
      .where(eq(schema.personToTag.personId, personId));
  }

  /**
   * Get all tags for a project
   */
  async getTagsForProject(projectId: string) {
    // Check if the project exists
    const [project] = await this.db
      .select()
      .from(schema.projects)
      .where(eq(schema.projects.id, projectId));
    
    if (!project) {
      throw new NotFoundException(`Project with ID ${projectId} not found`);
    }
    
    // Get all tags for the project
    return this.db
      .select({
        tag: schema.tags,
      })
      .from(schema.projectToTag)
      .innerJoin(schema.tags, eq(schema.projectToTag.tagId, schema.tags.id))
      .where(eq(schema.projectToTag.projectId, projectId));
  }
}